@@ -27,26 +27,29 @@
 
 #ifdef HAVE_POPPLER
 
-#include "Function.h"
-#include "GfxFont.h"
-#include "GfxState.h"
-#include "Page.h"
-#include "Stream.h"
+#include <poppler/Function.h>
+#include <poppler/GfxFont.h>
+#include <poppler/GfxState.h>
+#include <poppler/Page.h>
+#include <poppler/Stream.h>
+#include <poppler/goo/gmem.h>
+
 #include "color.h"
+#include "color/cms-util.h"
+#include "display/cairo-utils.h"
+#include "display/nr-filter-utils.h"
 #include "document.h"
 #include "extract-uri.h"
+#include "helper/geom.h"
+#include "object/sp-defs.h"
+#include "object/sp-item-group.h"
+#include "object/sp-namedview.h"
+#include "object/sp-text.h"
 #include "pdf-parser.h"
 #include "pdf-utils.h"
 #include "png.h"
 #include "poppler-cairo-font-engine.h"
 #include "profile-manager.h"
-
-#include "color/cms-util.h"
-#include "display/cairo-utils.h"
-#include "display/nr-filter-utils.h"
-#include "object/sp-defs.h"
-#include "object/sp-item-group.h"
-#include "object/sp-namedview.h"
 #include "svg/css-ostringstream.h"
 #include "svg/path-string.h"
 #include "svg/svg.h"
@@ -80,10 +83,6 @@ SvgBuilder::SvgBuilder(SPDocument *document, gchar *docname, XRef *xref)
     _xml_doc = _doc->getReprDoc();
     _container = _root = _doc->getReprRoot();
     _init();
-
-    // Set default preference settings
-    _preferences = _xml_doc->createElement("svgbuilder:prefs");
-    _preferences->setAttribute("embedImages", "1");
 }
 
 SvgBuilder::SvgBuilder(SvgBuilder *parent, Inkscape::XML::Node *root) {
@@ -92,7 +91,6 @@ SvgBuilder::SvgBuilder(SvgBuilder *parent, Inkscape::XML::Node *root) {
     _docname = parent->_docname;
     _xref = parent->_xref;
     _xml_doc = parent->_xml_doc;
-    _preferences = parent->_preferences;
     _container = this->_root = root;
     _init();
 }
@@ -206,7 +204,7 @@ void SvgBuilder::cropPage(const Geom::Rect &bbox)
     val << "M" << box.left() << " " << box.top()
         << "H" << box.right() << "V" << box.bottom()
         << "H" << box.left() << "Z";
-    auto clip_path = _createClip(val.str(), Geom::identity(), false);
+    auto clip_path = _createClip(val.str(), false);
     gchar *urltext = g_strdup_printf("url(#%s)", clip_path->attribute("id"));
     _container->setAttribute("clip-path", urltext);
     g_free(urltext);
@@ -350,11 +348,7 @@ void SvgBuilder::_addToContainer(Inkscape::XML::Node *node, bool release)
 void SvgBuilder::_setClipPath(Inkscape::XML::Node *node)
 {
     if (_clip_history->hasClipPath() || _clip_text) {
-        auto tr = Geom::identity();
-        if (auto attr = node->attribute("transform")) {
-            sp_svg_transform_read(attr, &tr);
-        }
-        if (auto clip_path = _getClip(tr)) {
+        if (auto clip_path = _getClip(node)) {
             gchar *urltext = g_strdup_printf("url(#%s)", clip_path->attribute("id"));
             node->setAttribute("clip-path", urltext);
             g_free(urltext);
@@ -715,6 +709,7 @@ void SvgBuilder::addPath(GfxState *state, bool fill, bool stroke, bool even_odd)
 
 void SvgBuilder::addClippedFill(GfxShading *shading, const Geom::Affine shading_tr)
 {
+    // Don't change to hasClipPath as it messes up gradient fills
     if (_clip_history->getClipPath()) {
         addShadedFill(shading, shading_tr, _clip_history->getClipPath(), _clip_history->getAffine(),
                       _clip_history->getClipType() == clipEO);
@@ -794,45 +789,110 @@ void SvgBuilder::setClip(GfxState *state, GfxClipType clip, bool is_bbox)
 /**
  * Return the active clip as a new xml node.
  */
-Inkscape::XML::Node *SvgBuilder::_getClip(const Geom::Affine &node_tr)
+Inkscape::XML::Node *SvgBuilder::_getClip(const Inkscape::XML::Node *node)
 {
     // In SVG the path-clip transforms are compounded, so we have to do extra work to
     // pull transforms back out of the clipping object and set them. Otherwise this
     // would all be a lot simpler.
+
+    // node_tr includes _page_affine
+    Geom::Affine node_tr = Geom::identity();
+    if (auto attr = node->attribute("transform")) {
+        sp_svg_transform_read(attr, &node_tr);
+    }
+
     if (_clip_text) {
-        auto node = _clip_text;
+        auto clip_node = _clip_text;
 
         auto text_tr = Geom::identity();
-        if (auto attr = node->attribute("transform")) {
+        if (auto attr = clip_node->attribute("transform")) {
             sp_svg_transform_read(attr, &text_tr);
-            node->removeAttribute("transform");
+            clip_node->removeAttribute("transform");
         }
 
-        for (auto child = node->firstChild(); child; child = child->next()) {
+        for (auto child = clip_node->firstChild(); child; child = child->next()) {
             Geom::Affine child_tr = text_tr * _page_affine * node_tr.inverse();
             svgSetTransform(child, child_tr);
         }
 
         _clip_text = nullptr;
-        return node;
+        return clip_node;
     }
-    if (_clip_history->hasClipPath()) {
-        std::string clip_d = svgInterpretPath(_clip_history->getClipPath());
-        Geom::Affine tr = _clip_history->getAffine() * _page_affine * node_tr.inverse();
-        return _createClip(clip_d, tr, _clip_history->evenOdd());
+    auto clip_pathv = _checkClip(node, node_tr);
+    if (!clip_pathv.empty()) {
+        // page and clip transforms are applied in _createClip, but we need to apply the
+        // node inverse so that it compounds properly when clipping in SVG
+        auto clip_d = sp_svg_write_path(clip_pathv * node_tr.inverse());
+        return _createClip(clip_d, _clip_history->evenOdd());
     }
     return nullptr;
 }
 
-Inkscape::XML::Node *SvgBuilder::_createClip(const std::string &d, const Geom::Affine tr, bool even_odd)
+Geom::PathVector SvgBuilder::_checkClip(const Inkscape::XML::Node *node, const Geom::Affine &node_tr) const
 {
+    Geom::PathVector empty_path;
+    if (!_clip_history->hasClipPath()) {
+        return empty_path;
+    }
+
+    // node_tr includes _page_affine, so we need to apply it to the clipping path
+    Geom::PathVector clip_pathv = getPathV(_clip_history->getClipPath()) * _clip_history->getAffine() * _page_affine;
+
+    // if this is a clipping or masking group situation, just return the clip
+    if (strcmp(node->name(), "svg:g") == 0 || _clip_groups > 0 || _mask_groups.size() > 0) {
+        return clip_pathv;
+    }
+
+    // Calculate bounding boxes for both the node and the clip path
+    Geom::PathVector node_vec = sp_svg_read_pathv(node->attribute("d"));
+
+    if (node_vec.empty()) {
+        // Non-path node (text, image, etc)
+        // Create a PathVector of the bounding box instead
+        _doc->ensureUpToDate();
+        auto item = cast<SPItem>(_doc->getObjectByRepr(const_cast<Inkscape::XML::Node *>(node)));
+        // transform will be applied later, so default identity is good
+        Geom::OptRect bounds;
+        bounds = item ? item->visualBounds() : bounds;
+
+        if (!bounds.empty()) {
+            node_vec.push_back(Geom::Path(*bounds));
+        } else {
+            // what could this be?? clip it just in case
+            return clip_pathv;
+        }
+    }
+
+    node_vec *= node_tr;
+
+    if (pathv_fully_contains(clip_pathv, node_vec, _clip_history->fillRule())) {
+        return empty_path;
+    } else {
+        return clip_pathv;
+    }
+}
+
+Inkscape::XML::Node *SvgBuilder::_createClip(const std::string &d, bool even_odd)
+{
+    if (_prev_clip) {
+        // Check if the previous clipping path would be identical to the new one.
+        auto prev_path = _prev_clip->firstChild();
+        auto prev_d = prev_path->attribute("d");
+        auto prev_even_odd =
+            prev_path->attribute("clip-rule") ? std::string(prev_path->attribute("clip-rule")) == "evenodd" : false;
+
+        // Don't create an identical new clipping path
+        if (prev_d == d && prev_even_odd == even_odd) {
+            return _prev_clip;
+        }
+    }
+
     Inkscape::XML::Node *clip_path = _xml_doc->createElement("svg:clipPath");
     clip_path->setAttribute("clipPathUnits", "userSpaceOnUse");
 
     // Create the path
     Inkscape::XML::Node *path = _xml_doc->createElement("svg:path");
     path->setAttribute("d", d);
-    svgSetTransform(path, tr);
 
     if (even_odd) {
         path->setAttribute("clip-rule", "evenodd");
@@ -843,6 +903,10 @@ Inkscape::XML::Node *SvgBuilder::_createClip(const std::string &d, const Geom::A
     // Append clipPath to defs and get id
     _doc->getDefs()->getRepr()->appendChild(clip_path);
     Inkscape::GC::release(clip_path);
+
+    // update the previous clip path
+    _prev_clip = clip_path;
+
     return clip_path;
 }
 
@@ -1964,14 +2028,12 @@ Inkscape::XML::Node *SvgBuilder::_createImage(Stream *str, int width, int height
         png_destroy_write_struct(&png_ptr, &info_ptr);
         return nullptr;
     }
-    // Decide whether we should embed this image
-    bool embed_image = _preferences->getAttributeBoolean("embedImages", true);
 
     // Set read/write functions
     std::vector<guchar> png_buffer;
     FILE *fp = nullptr;
     gchar *file_name = nullptr;
-    if (embed_image) {
+    if (_embed_images) {
         png_set_write_fn(png_ptr, &png_buffer, png_write_vector, nullptr);
     } else {
         static int counter = 0;
@@ -2032,7 +2094,11 @@ Inkscape::XML::Node *SvgBuilder::_createImage(Stream *str, int width, int height
         } else {
             image_stream = new ImageStream(str, width, 1, 1);
         }
+#if POPPLER_CHECK_VERSION(26, 0, 0)
+        image_stream->rewind();
+#else
         image_stream->reset();
+#endif
 
         // Convert grayscale values
         unsigned char *buffer = new unsigned char[width];
@@ -2058,7 +2124,11 @@ Inkscape::XML::Node *SvgBuilder::_createImage(Stream *str, int width, int height
         image_stream = new ImageStream(str, width,
                                        color_map->getNumPixelComps(),
                                        color_map->getBits());
+#if POPPLER_CHECK_VERSION(26, 0, 0)
+        image_stream->rewind();
+#else
         image_stream->reset();
+#endif
 
         // Convert RGB values
         unsigned int *buffer = new unsigned int[width];
@@ -2096,7 +2166,7 @@ Inkscape::XML::Node *SvgBuilder::_createImage(Stream *str, int width, int height
 
     } else {    // A colormap must be provided, so quit
         png_destroy_write_struct(&png_ptr, &info_ptr);
-        if (!embed_image) {
+        if (!_embed_images) {
             fclose(fp);
             g_free(file_name);
         }
@@ -2124,7 +2194,7 @@ Inkscape::XML::Node *SvgBuilder::_createImage(Stream *str, int width, int height
     image_node->setAttribute("preserveAspectRatio", "none");
 
     // Create href
-    if (embed_image) {
+    if (_embed_images) {
         // Append format specification to the URI
         auto *base64String = g_base64_encode(png_buffer.data(), png_buffer.size());
         auto png_data = std::string("data:image/png;base64,") + base64String;
